module("luci.controller.modbus", package.seeall)

local uci = require "luci.model.uci".cursor()
local utl = require "luci.util"
local sys = require "luci.sys"

function index()
	local modbus = nixio.fs.access("/usr/bin/modbus-master")
	local modbus_to_ftp = nixio.fs.access("/usr/bin/modbus-to-ftp")
	local modbus_to_mqtt = nixio.fs.access("/usr/bin/modbus-to-mqtt")
	local modbus_to_iec104 = nixio.fs.access("/usr/bin/modbus-to-iec104")
	local modbus_to_dnp3 = nixio.fs.access("/usr/bin/modbus-to-dnp3")
	if modbus then
		entry({"admin", "services", "modbus"},  firstchild(), _("Modbus"), 97)
		entry({"admin", "services", "modbus", "modbus_master"}, arcombine(cbi("modbus_master"),
			cbi("slave_details")), _("Modbus Master"), 30).leaf = true
		entry({"admin", "services", "modbus", "query_mapping"}, cbi("query_mapping"), nil, 40).leaf = true
	end
	if modbus_to_ftp then
		entry({"admin", "services", "modbus", "modbus_to_ftp"},  cbi("modbus_to_ftp"),  _("Modbus to FTP"), 50).leaf = true
	end
	if modbus_to_mqtt then
		entry({"admin", "services", "modbus", "modbus_to_mqtt"},  cbi("modbus_to_mqtt"),  _("Modbus to MQTT"), 60).leaf = true
	end
	if modbus_to_iec104 then
		entry({"admin", "services", "modbus", "modbus_to_iec104"},  cbi("modbus_to_iec104"),  _("Modbus to IEC104"), 70).leaf = true
	end
	if modbus_to_dnp3 then
		entry({"admin", "services", "modbus", "modbus_to_dnp3"},  cbi("modbus_to_dnp3"),  _("Modbus to DNP3"), 80).leaf = true
	end

	entry({"admin", "services", "modbus", "backup"},  cbi("backup"),  _("Backup and Restore"), 90).leaf = true

end



function copy_section(section_type, old_config, old_section, new_config, new_section)
	uci:foreach(old_config, section_type, function(s)
		if s[".name"] == old_section then
			for key, value in pairs(s) do
				if key and value and key ~= ".name" and key ~= ".index" then
					if type(value) == "table" then
						uci:set(new_config, new_section, tostring(key), value)
					else
						uci:set(new_config, new_section, tostring(key), tostring(value))
					end
				end
			end
		end
	end)
end

function test_serial_request()
	local arg = {}
	arg[1] = "/usr/sbin/modbus_serial_request_test"
	arg[2] = "'" .. luci.http.formvalue("rstype"):gsub("'", "") .. "'"
	arg[3] = "'" .. luci.http.formvalue("timeout"):gsub("'", "") .. "'"
	arg[4] = "'" .. luci.http.formvalue("slave_id"):gsub("'", "") .. "'"
	arg[5] = "'" .. luci.http.formvalue("fc"):gsub("'", "") .. "'"
	arg[6] = "'" .. luci.http.formvalue("regaddr"):gsub("'", "") .. "'"
	arg[7] = "'" .. luci.http.formvalue("payload"):gsub("'", "") .. "'"
	arg[8] = "'" .. luci.http.formvalue("datatype"):gsub("'", "") .. "' 2>&1"
	local cmd = table.concat(arg, " ")

	luci.http.prepare_content("text/plain")
	local util = io.popen(cmd)
	if util then
		while true do
			local ln = util:read("*l")
			if not ln then break end
			luci.http.write(ln .. "\n")
		end
		util:close()
	else
		luci.http.write("Test failed")
	end
end

function test_request()
	local arg = {}
	arg[1] = "/usr/sbin/modbus_tcp_test"
	arg[2] = "'" .. luci.http.formvalue("ip"):gsub("'", "") .. "'"
	arg[3] = "'" .. luci.http.formvalue("port"):gsub("'", "") .. "'"
	arg[4] = "'" .. luci.http.formvalue("timeout"):gsub("'", "") .. "'"
	arg[5] = "'" .. luci.http.formvalue("slave_id"):gsub("'", "") .. "'"
	arg[6] = "'" .. luci.http.formvalue("fc"):gsub("'", "") .. "'"
	arg[7] = "'" .. luci.http.formvalue("regaddr"):gsub("'", "") .. "'"
	arg[8] = "'" .. luci.http.formvalue("payload"):gsub("'", "") .. "'"
	arg[9] = "'" .. luci.http.formvalue("datatype"):gsub("'", "") .. "' 2>&1"
	local cmd = table.concat(arg, " ")

	luci.http.prepare_content("text/plain")
	local util = io.popen(cmd)
	if util then
		while true do
			local ln = util:read("*l")
			if not ln then break end
			luci.http.write(ln .. "\n")
		end
		util:close()
	else
		luci.http.write("Test failed")
	end
end
