local uci = require "luci.model.uci".cursor()
local ds = require "luci.dispatcher"
local sys = require "luci.sys"

m = Map("modbus-master", "", "")

section_global = m:section(TypedSection, "global", translate("Enable/ Disable"))
section_global.anonymous = true
section_global.addremove = false
section_global.name = "global"

option_enabled = section_global:option(Flag, "enabled", translate("Enabled"),
		translate("Enable Modbus Master"))
option_enabled.rmempty=false
option_enabled.default="1"


s = m:section(TypedSection, "modbus_device", translate("Modbus Slave devices"))
s.add_toggled = true
s.addremove = true
s.anonymous = true
s.sortable = false
s.dont_show_add = true
s.novaluetext = translate("No Modbus slaves added yet")
s.template = "cbi/tblsection"

function s.extedit(self, section)
	local connection_id = self.map.uci:get("modbus-master", section, "connection_id")
	connection_id = (type(connection_id) == "string" and connection_id or "")
	return ds.build_url("admin", "services", "modbus", "modbus_master", connection_id)
end

function s.create(self, section, ...)
	local connection_id = sys.exec("head -c 32 /dev/urandom | md5sum | awk '{ print $1 }' | tr -d '\n'")
	local created = TypedSection.create(self, section, ...)
	self.map:set(created, "connection_id", connection_id)

	if created then
		luci.http.redirect(ds.build_url("admin/services/modbus/modbus_master", connection_id))
	end
end

function s.remove(self, section)
	--when deleting slave device, also delete all configured registers
	if (section) then
		local connection_id = self.map.uci:get(self.config, section, "connection_id")
		local query_id = self.map.uci:get(self.config, section, "query_id")

		if (connection_id) then
			m.uci:delete_all("modbus-master", "modbus_query",
					function(s)
						if(s.connection == connection_id) then
							query_id = s.query_id
							if query_id then
								m.uci:delete_all("modbus-master", "modbus_register",
										function(s) return s.query_id == query_id end)
							end
						end

						return s.connection == connection_id end)
		end
	end

	m.uci:delete("modbus-master", section)
	m.uci:save("modbus-master")
	m.uci:commit("modbus-master")

	luci.http.redirect(ds.build_url("admin/services/modbus/modbus_master"))
end

name = s:option(DummyValue, "name", translate("Name"))
function name.cfgvalue(self, s)
	local set_name = self.map:get(s, "conf_name")
	if set_name then
		if #set_name < 24 then
			return set_name
		else
			return string.sub(set_name,1,24) .. "..."
		end
	else
		return "Nameless device"
	end
end

serial_port = s:option(DummyValue, "serial_port", translate("Serial Port"))
function serial_port.cfgvalue(self, s)
	return self.map:get(s, "serial_port") or "N/A"
end

host = s:option(DummyValue, "host", translate("Host"))
function host.cfgvalue(self, s)
	local set_ip = self.map:get(s, "host")
	if set_ip then
		if #set_ip < 24 then
			return set_ip
		else
			return string.sub(set_ip,1,24) .. "..."
		end
	else
		return "N/A"
	end
end

port = s:option(DummyValue, "port", translate("Port"))
function port.cfgvalue(self, s)
	return self.map:get(s, "port") or "N/A"
end

enabled = s:option(Flag, "enabled")
enabled.last = true

--Queries--
s2 = m:section(TypedSection, "modbus_query", "Queries")
s2.anonymous = true
s2.template = "cbi/tblsection"

name = s2:option(DummyValue, "name", translate("Query Name"))
function name.cfgvalue(self, s)
	return self.map:get(s, "name") or "N/A"
end

f_code = s2:option(DummyValue, "function", translate("Function"))
function f_code.cfgvalue(self, s)
	return self.map:get(s, "function") or "N/A"
end


start_address = s2:option(DummyValue, "start_address", translate("Start Address"))
function start_address.cfgvalue(self, s)
	return self.map:get(s, "start_address") or "N/A"
end

reg_count = s2:option(DummyValue, "reg_count", translate("Count"))
function reg_count.cfgvalue(self, s)
	return self.map:get(s, "reg_count") or "N/A"
end

qslave_id = s2:option(DummyValue, "slave_id", translate("Slave ID"))
function qslave_id.cfgvalue(self, s)
	return self.map:get(s, "slave_id") or "N/A"
end

polling_interval = s2:option(DummyValue, "polling_interval", translate("Polling Interval"))
function polling_interval.cfgvalue(self, s)
	return self.map:get(s, "polling_interval") or "N/A"
end

--Registers--
s3 = m:section(TypedSection, "modbus_register", "Registers")
s3.anonymous = true
s3.template = "cbi/tblsection"

address = s3:option(DummyValue, "address", translate("Address"), translate(""))
function address.cfgvalue(self, s)
	return self.map:get(s, "address") or "N/A"
end

name = s3:option(DummyValue, "name", translate("Register Name"), translate(""))
function name.cfgvalue(self, s)
	return self.map:get(s, "name") or "N/A"
end

data_type = s3:option(DummyValue, "data_type", translate("Dataype"))
function data_type.cfgvalue(self, s)
	return self.map:get(s, "data_type") or "N/A"
end

scaling_a = s3:option(DummyValue, "scaling_a", translate("Scaling A"), translate(""))
function scaling_a.cfgvalue(self, s)
	return self.map:get(s, "scaling_a") or "N/A"
end

scaling_b = s3:option(DummyValue, "scaling_b", translate("Scaling B"), translate(""))
function scaling_b.cfgvalue(self, s)
	return self.map:get(s, "scaling_b") or "N/A"
end

local modbus_to_iec104 = nixio.fs.access("/usr/bin/modbus-to-iec104")
if modbus_to_iec104 then
	ioa_type = s3:option(DummyValue, "ioa_type", "IOA Type")
	function ioa_type.cfgvalue(self, s)
		return self.map:get(s, "ioa_type") or "N/A"
	end

	ioa_address = s3:option(DummyValue, "ioa_address", "IOA Address")
	function ioa_address.cfgvalue( self,s )
		return self.map:get(s, "ioa_address") or "N/A"
	end
end

local dnp3 = nixio.fs.access("/usr/bin/modbus-to-dnp3")
if dnp3 then
	dnp3_static_variation = s3:option(DummyValue, "dnp3_static_variation", "Default Static Variation")
	function dnp3_static_variation.cfgvalue(self, s)
		return self.map:get(s, "dnp3_static_variation") or "N/A"
	end

	dnp3_event_variation = s3:option(DummyValue, "dnp3_event_variation", "Default Event Variation")
	function dnp3_event_variation.cfgvalue(self, s)
		return self.map:get(s, "dnp3_event_variation") or "N/A"
	end

	class = s3:option(DummyValue, "dnp3_class", translate("Class"))
	function class.cfgvalue(self, s)
		return self.map:get(s, "class") or "N/A"
	end

	dnp3_index = s3:option(DummyValue, "dnp3_index", "Index")
	function dnp3_index.cfgvalue(self, s)
		return self.map:get(s, "dnp3_index") or "N/A"
	end
end

-- process_or_config = s3:option(DummyValue, "process_or_config", translate("Process Or Config"))
-- function process_or_config.cfgvalue(self, s)
-- 	return self.map:get(s, "process_or_config") or "N/A"
-- end

hi_threashold = s3:option(DummyValue, "hi_threashold", translate("High Threashold"), translate(""))
function hi_threashold.cfgvalue(self, s)
	return self.map:get(s, "hi_threashold") or "N/A"
end

low_threashold = s3:option(DummyValue, "low_threashold", translate("Low Threashold"), translate(""))
function low_threashold.cfgvalue(self, s)
	return self.map:get(s, "low_threashold") or "N/A"
end

send_alarm = s3:option(DummyValue, "send_alarm", translate("Send Alarm"))
function send_alarm.cfgvalue(self, s)
	return self.map:get(s, "send_alarm") or "N/A"
end
return m
