local uci = require "luci.model.uci".cursor()
local ds = require "luci.dispatcher"
local wa = require "luci.tools.webadmin"
local sys = require "luci.sys"
local fs = require "nixio.fs"

--DNP3 TO MODBUS Configuration--
m = Map("modbus-to-dnp3", translate("Configuration"))
section_global = m:section(TypedSection, "global", translate("Enable/ Disable"))
section_global.anonymous = true
section_global.addremove = false
section_global.name = "global"

option_enabled = section_global:option(Flag, "enabled", translate("Enabled"),
    translate("Enable Modbus to DNP3"))
option_enabled.rmempty = false
option_enabled.default = "0"

s = m:section(TypedSection, "modbus_to_dnp3", "Modbus to DNP3")
s.anonymous = true

rtu_dnp_address = s:option(Value, "LocalAddr", "Outstation Address", translate("DNP3 address of the local device"))
rtu_dnp_address.datatype = "range(1,1024)"
rtu_dnp_address.default = 1;

dnp_master_address = s:option(Value, "RemoteAddr", "DNP Master Address",translate("DNP3 address of the remote device"))
dnp_master_address.datatype = "range(1,1024)"
dnp_master_address.default = 2

dl_max_frame_size = s:option(Value, "KeepAliveTimeout", "Keep_Alive_Timeout",translate("The interval (In Minutes) for keep-alive messages (link status requests)"))
dl_max_frame_size.default = 1

dl_retries = s:option(Value, "Timeout", "Timeout",translate("The response timeout in milliseconds for confirmed requests"))
dl_retries.default = 1000

maxBinaryEvents = s:option(Value, "maxBinaryEvents", "Max Binary Events",translate("The number of binary events the outstation will buffer before overflowing"))
maxBinaryEvents.datatype = "range(1,100)"

maxDoubleBinaryEvents = s:option(Value, "maxDoubleBinaryEvents", "Max Double Binary Events",translate("The number of double binary events the outstation will buffer before overflowing"))
maxDoubleBinaryEvents.datatype = "range(1,100)"

maxAnalogEvents = s:option(Value, "maxAnalogEvents", "Max Analog Events",translate("The number of analog events the outstation will buffer before overflowing"))
maxAnalogEvents.datatype = "range(1,100)"

maxBinaryOutputStatusEvents = s:option(Value, "maxBinaryOutputStatusEvents", "Max Binary Output Status Events",translate("The number of binary output status events the outstation will buffer before overflowing"))
maxBinaryOutputStatusEvents.datatype = "range(1,100)"


maxAnalogOutputStatusEvents = s:option(Value, "maxAnalogOutputStatusEvents", "Max Analog Output Status Events",translate("The number of binary output status events the outstation will buffer before overflowing"))
maxAnalogOutputStatusEvents.datatype = "range(1,100)"

allowUnsolicited = s:option(ListValue, "allowUnsolicited", "Allow Unsolicited",translate("Enabled / Disable unsolicited messages"))
allowUnsolicited:value("1",translate("Enable"))
allowUnsolicited:value("0",translate("Disable"))

unsolClassMask = s:option(ListValue, "unsolClassMask","Unsolicited Class Mask", translate("Class mask for unsolicted"))
unsolClassMask:value("AllEventClasses", translate("AllEventClasses"))
unsolClassMask:value("Class1", translate("Class1"))
unsolClassMask:value("Class2", translate("Class2"))
unsolClassMask:value("Class3", translate("Class3"))
unsolClassMask.default = "AllEventClasses"

maxTxFragSize = s:option(Value, "maxTxFragSize", "MaxTxFragSize",translate("The maximum fragment size the outstation will use for fragments it sends"))
maxTxFragSize.datatype = "integer"
maxTxFragSize.default = 2048

maxRxFragSize = s:option(Value, "maxRxFragSize", "MaxRxFragSize",translate("The maximum fragment size the outstation will be able to receive"))
maxRxFragSize.datatype = "integer"
maxRxFragSize.default = 2048


numUnsolRetries = s:option(Value, "numUnsolRetries","Unsol. Retries", translate("Number of unsolicited non-regenerated retries"))
numUnsolRetries.datatype = "integer"
numUnsolRetries.default = 100

unsolConfirmTimeout = s:option(Value, "unsolConfirmTimeout", "Timeout for unsolicited confirms (Seconds)")
unsolConfirmTimeout.datatype = "integer"
unsolConfirmTimeout.default = 5

solConfirmTimeout = s:option(Value, "solConfirmTimeout", "Timeout for solicited confirms (Seconds)")
solConfirmTimeout.datatype = "integer"
solConfirmTimeout.default = 5


selectTimeout = s:option(Value, "selectTimeout", "Select Timeout",translate("How long the outstation will allow an operate to proceed after a prior select"))
selectTimeout.default = 10

maxControlsPerRequest = s:option(Value, "maxControlsPerRequest", "Max controls per request",translate("The maximum number of controls the outstation will attempt to process from a single APDU"))
maxControlsPerRequest.datatype = "integer"
maxControlsPerRequest.default = 16

respondToAnyMaster = s:option(ListValue, "respondToAnyMaster", "Respond to any master",translate("If true, the outstation processes responds to any request/confirmation as if it came from the expected master address"))
respondToAnyMaster:value("1",translate("True"))
respondToAnyMaster:value("0",translate("False"))
respondToAnyMaster.default = 0


--IOA 
m1 = Map("modbus-master", translate(""))

s2 = m1:section(TypedSection, "modbus_register", "IOA")
s2.template = "cbi/tblsection"
s2.anonymous = true

name = s2:option(DummyValue, "name", "Name")

address = s2:option(DummyValue, "address", "Address")

data_type = s2:option(DummyValue, "data_type", "Data Type")

dnp3_static_variation = s2:option(DummyValue, "dnp3_static_variation", "Default Static Variation")
function dnp3_static_variation.cfgvalue(self, s)
    return self.map:get(s, "dnp3_static_variation") or "N/A"
end

dnp3_event_variation = s2:option(DummyValue, "dnp3_event_variation", "Default Event Variation")
function dnp3_event_variation.cfgvalue(self, s)
    return self.map:get(s, "dnp3_event_variation") or "N/A"
end

class = s2:option(DummyValue, "dnp3_class", translate("Class"))
function class.cfgvalue(self, s)
    return self.map:get(s, "class") or "N/A"
end

dnp3_index = s2:option(DummyValue, "dnp3_index", "Index")
function dnp3_index.cfgvalue(self, s)
    return self.map:get(s, "dnp3_index") or "N/A"
end

return m, m1