local fs = require "nixio.fs" 

m = Map("modbus-to-iec104", translate("Modbus to IEC104"))

section_global = m:section(TypedSection, "global", translate("Enable/ Disable"))
section_global.anonymous = false
section_global.addremove = false
section_global.name = "global"

option_enabled = section_global:option(Flag, "enabled", translate("Enabled"),
	translate("Enable Modbus to IEC104"))
option_enabled.rmempty=false
option_enabled.default="1"

s = m:section(TypedSection, "modbus_to_104", translate("Modbus to IEC104"), "")
s.anonymous = true
s.addremove = false


cyclic_time_interval = s:option(Value, "cyclic_time_interval", "Cyclic Time Interval", translate("Set time interval to send (Unsolicited) ASDU with COT as a Cyclic"))
cyclic_time_interval.datatype = "integer"

k = s:option(Value, "k", "K", translate("Maximum unacknowledged transmitted APDUs"))
k.datatype = "integer"
k.default = "12"

w = s:option(Value, "w", "W", translate("Maximum unacknowledged received APDUs"))
w.datatype = "integer"
w.default = "8"

t0 = s:option(Value, "t0", "T0", translate("Amount of time that the IEC60870 Master will attempt to connect to the device"))
t0.datatype = "range(1,30)"
t0.default = "3"

t1 = s:option(Value, "t1", "T1", translate("Time to wait for a acknowledgment (“ACK”) to a transmitted APDU"))
t1.datatype = "integer"
t1.default = "15"

t2 = s:option(Value, "t2", "T2", translate("Time to wait before sending a supervisory APDU ACK"))
t2.datatype = "integer"
t2.default = "10"

t3 = s:option(Value, "t3", "T3", translate("Idle time before sending TEST APDU"))
t3.datatype = "integer"
t3.default = "20"

sizeof_cot = s:option(Value, "sizeof_cot", "Size Of COT", translate("Specify the number of octets in an ASDU COT field"))
sizeof_cot.datatype = "range(1,2)"
sizeof_cot.default = "2"

oa = s:option(Value, "oa", "OA", translate("This is required when there is more than one controlling station(Master/Client)"))
oa.datatype = "range(0,254)"
oa.default = "0"

ca = s:option(Value, "ca", "CA", translate("Common Address or ASDU Address"))
ca.datatype = "range(0,254)"
ca.default = "1"


sizeof_ca = s:option(ListValue, "sizeof_ca", "Size Of CA", translate("Size Of Common Address or ASDU Address for IEC101 is 1 and for  IEC104 is 2"))
sizeof_ca:value("1",translate("1"))
sizeof_ca:value("2",translate("2"))
sizeof_ca.default = "2"

sizeof_ioa = s:option(ListValue, "sizeof_ioa", "Size Of IOA", translate("Size Of Information Object Address (IOA) (1/2/3 - default = 3)"))
sizeof_ioa:value("1",translate("1"))
sizeof_ioa:value("2",translate("2"))
sizeof_ioa:value("3",translate("3"))
sizeof_ioa.default = "3"

max_asdu_length = s:option(ListValue, "max_asdu_length", "Max. ASDU length", translate("Maximum size of ASDU that is generated,Its value is 249 for IEC104 and 254 for IEC101."))
max_asdu_length:value("249",translate("249"))
max_asdu_length:value("254",translate("254"))

return m
