local fs = require "nixio.fs"

m = Map("modbus-to-mqtt", translate("Modbus to MQTT"))

section_global = m:section(TypedSection, "global", translate("Enable/ Disable"))
section_global.anonymous = false
section_global.addremove = false
section_global.name = "global"

option_enabled = section_global:option(Flag, "enabled", translate("Enabled"),
    translate("Enable Modbus to MQTT"))
option_enabled.rmempty = false
option_enabled.default = "1"

s = m:section(TypedSection, "modbus_mqtt", translate("Modbus MQTT"), "")
s.anonymous = true
s.addremove = false


host = s:option(Value, "host", "Host", translate("MQTT broker hostname or IP"))
host.datatype = "host"
host.default = "127.0.0.1"

port = s:option(Value, "port", "Port", translate("MQTT broker port number"))
port.datatype = "port"
port.default = "1883"

user = s:option(Value, "user", "Username", translate("MQTT client username (leave empty if anonymous)"))
user.datatype = "uciname"

pass = s:option(Value, "pass", "Password", translate("MQTT client password (leave empty if anonymous)"))
pass.password = true
--pass.datatype = "regex('^[a-zA-Z0-9!@#$%%&*+/=?^_`{|}~.-]+$',5)"
function pass:parse(section, novld, ...)
    local u = user:formvalue(section)
    local p = pass:formvalue(section)
    local uEmpty = (u == nil or u == "")
    local pEmpty = (p == nil or p == "")
    if (uEmpty and not pEmpty) or (not uEmpty and pEmpty) then
        self:add_error(section, "invalid", "Both username and password must be either empty or filled")
    end
    Value.parse(self, section, novld, ...)
end

client_id = s:option(Value, "client_id", "Client Id", translate("Leave blank to generate random client id on connection"))
client_id.datatype = "string"


qos = s:option(Value, "qos", "QoS (Quality Of Service)")
qos.datatype = "range(0,2)"

clean_session = s:option(ListValue, "clean_session", "Clean Session", translate("Tells MQTT Broker if it should discard existing session"))
clean_session:value("1", translate("Yes"))
clean_session:value("0", translate("No"))

retain = s:option(ListValue, "retain", "Retain", translate("Makes the message retained."))
retain:value("1", translate("Yes"))
retain:value("0", translate("No"))
retain.default = "0"

--TODO add translate and labels
will_topic = s:option(Value, "data_topic", translate("Data Topic"), "Topic to send Modbus Data")
will_topic.datatype = "string"

will_topic = s:option(Value, "command_topic", "Command Topic", translate("Topic on which Modbus commands are received"))
will_topic.datatype = "string"

will_topic = s:option(Value, "command_response_topic", "Command Response Topic", translate("Topic on which Modbus command responses are sent"))
will_topic.datatype = "string"

will_topic = s:option(Value, "will_topic", "Will Topic")
will_topic.datatype = "string"

will_payload = s:option(Value, "will_payload", "Will Payload", translate("Payload to publish in case connection is interrupted"))
will_payload.datatype = "string"

will_qos = s:option(Value, "will_qos", "Will QoS")
will_qos.datatype = "range(1,10)"

will_retain = s:option(ListValue, "will_retain", "Will Retain", translate("Should will message be retained"))
will_retain:value("1", translate("Yes"))
will_retain:value("0", translate("No"))

reconnect_delay = s:option(Value, "reconnect_delay", "Reconnect Delay", translate("Number of seconds to wait before retrying to connect broken connection"))
reconnect_delay.datatype = "range(1,900)"
reconnect_delay.default = 5

reconnect_delay_max = s:option(Value, "reconnect_delay_max", "Reconnect Delay Max", translate("Maximum number of seconds to wait before retrying"))
reconnect_delay_max.datatype = "range(1,3600)"
reconnect_delay_max.default = 900

reconnect_exponential_backoff = s:option(ListValue, "reconnect_exponential_backoff", "Use Exponential Backoff", translate("Should exponential backoff strategy is used when retrying"))
reconnect_exponential_backoff:value("1", translate("Yes"))
reconnect_exponential_backoff:value("0", translate("No"))
reconnect_exponential_backoff.default = "0"

max_inflight_messages = s:option(Value, "max_inflight_messages", "Maximum Inflight Messages", translate("Maximum inflight messages"))
max_inflight_messages.datatype = "range(1,100)"
max_inflight_messages.default = 20
return m
