local uci = require "luci.model.uci".cursor()
local ds = require "luci.dispatcher"
local wa = require "luci.tools.webadmin"
local sys = require "luci.sys"
local fs = require "nixio.fs"

local query_id
if arg[1] then
    query_id = arg[1]
end

m = Map("modbus-master", translate("Query Register Mapping"))

local cfgName
m.uci:foreach("modbus-master", "modbus_query", function(s)
    if s.query_id == query_id then
        cfgName = s[".name"]
    end
end)

s = m:section(NamedSection, cfgName, "Query", "Modbus Query")
--s.template = "modbus_master/tblsection"

function s:filter(section)
    local s_query_id = self.map.uci:get(self.config, section, "query_id")
    if (s_query_id == query_id) then
        return section
    else
        return nil
    end
end

name = s:option(Value, "name", translate("Query Name"))
name.optional = false

f_code = s:option(ListValue, "function", translate("Modbus Function"))
f_code:value("1", translate("Read coils (1)"))
f_code:value("2", translate("Read input coils (2)"))
f_code:value("3", translate("Read holding registers (3)"))
f_code:value("4", translate("Read input registers (4)"))
f_code.default = "3"

start_address = s:option(Value, "start_address", translate("Start Address"))
start_address.datatype = "range(0,65536)"
start_address.default = "1"
start_address.rmempty = false

reg_count = s:option(Value, "reg_count", translate("Count"))
reg_count.default = "1"
reg_count.rmempty = false
reg_count.validator_hint = translate("Single or multiple integers allowed.")

slave_id = s:option(Value, "slave_id", translate("Slave Id"))
slave_id.default = "1"
slave_id.datatype = "range(1,255)"

polling_interval = s:option(Value, "polling_interval", translate("Polling Interval (Seconds)"))
polling_interval.datatype = "range(1,65536)"
polling_interval.default = "60"
polling_interval.rmempty = false

enabled = s:option(Flag, "enabled", translate("Enabled"))
enabled.default = true


s = m:section(TypedSection, "modbus_register", translate("Modbus Registers"))
--s.template = "cbi/tblsection"
s.addremove = true
s.anonymous = true
s.add_toggled = true
s.novaluetext = translate("No registers configured for this slave device yet")

function s:filter(section)
    local s_query_id = self.map.uci:get(self.config, section, "query_id")
    if (s_query_id == query_id) then
        return section
    else
        return nil
    end
end

function s.create(self, section, ...)
    local register_id = sys.exec("head -c 32 /dev/urandom | md5sum | awk '{ print $1 }' | tr -d '\n'")
    local created = TypedSection.create(self, section, ...)
    self.map:set(created, "register_id", register_id)
    self.map:set(created, "query_id", query_id)

    if created then
        luci.http.redirect(ds.build_url("admin/services/modbus/query_mapping", query_id))
    end
end

address = s:option(Value, "address", translate("Address"), translate(""))
address.datatype = "integer"

name = s:option(Value, "name", translate("Name"), translate(""))
name.datatype = "string"

data_type = s:option(ListValue, "data_type", translate("Dataype"))
data_type:value("bit", translate("Bit (Coils/Inputs)"))

data_type:value("int", translate("Integer"))
data_type:value("uint", translate("Unsigned Integer"))

data_type:value("float_abcd", translate("32bit float, Byte order 1,2,3,4"))
data_type:value("float_dcba", translate("32bit float, Byte order 4,3,2,1"))
data_type:value("float_badc", translate("32bit float, Byte order 2,1,4,3"))
data_type:value("float_cdab", translate("32bit float, Byte order 3,4,1,2"))

data_type:value("int32_abcd", translate("32bit int, Byte order 1,2,3,4"))
data_type:value("int32_dcba", translate("32bit int, Byte order 4,3,2,1"))
data_type:value("int32_badc", translate("32bit int, Byte order 2,1,4,3"))
data_type:value("int32_cdab", translate("32bit int, Byte order 3,4,1,2"))

data_type:value("int64_abcdefgh", translate("64bit int, Byte order 1,2,3,4,5,6,7,8"))
data_type:value("int64_hgfedcba", translate("64bit int, Byte order 8,7,6,5,4,3,2,1"))
data_type:value("int64_badcfehg", translate("64bit int, Byte order 2,1,4,3,6,5,8,7"))
data_type:value("int64_ghefcdab", translate("64bit int, Byte order 7,8,5,6,3,4,1,2"))

data_type:value("uint32_abcd", translate("32bit Unsigned int, Byte order 1,2,3,4"))
data_type:value("uint32_dcba", translate("32bit Unsigned int, Byte order 4,3,2,1"))
data_type:value("uint32_badc", translate("32bit Unsigned int, Byte order 2,1,4,3"))
data_type:value("uint32_cdab", translate("32bit Unsigned int, Byte order 3,4,1,2"))

data_type:value("uint64_abcdefgh", translate("64bit Unsigned int, Byte order 1,2,3,4,5,6,7,8"))
data_type:value("uint64_hgfedcba", translate("64bit Unsigned int, Byte order 8,7,6,5,4,3,2,1"))
data_type:value("uint64_badcfehg", translate("64bit Unsigned int, Byte order 2,1,4,3,6,5,8,7"))
data_type:value("uint64_ghefcdab", translate("64bit Unsigned int, Byte order 7,8,5,6,3,4,1,2"))

data_type:value("float64_abcdefgh", translate("64bit float, Byte order 1,2,3,4,5,6,7,8"))
data_type:value("float64_hgfedcba", translate("64bit float, Byte order 8,7,6,5,4,3,2,1"))
data_type:value("float64_badcfehg", translate("64bit float, Byte order 2,1,4,3,6,5,8,7"))
data_type:value("float64_ghefcdab", translate("64bit float, Byte order 7,8,5,6,3,4,1,2"))

data_type.default = "none"
data_type.rmempty = false

scaling_a = s:option(Value, "scaling_a", translate("Scaling A"), translate(""))
scaling_a.datatype = "float"
scaling_a.default = 1

scaling_b = s:option(Value, "scaling_b", translate("Scaling B"), translate(""))
scaling_b.datatype = "float"
scaling_b.default = 0

-- process_or_config = s:option(ListValue, "process_or_config", translate("Process Or Config"))
-- process_or_config:value("process", translate("Process"))
-- process_or_config:value("config", translate("Config"))
-- process_or_config.default = "none"
-- process_or_config.rmempty = false

hi_threashold = s:option(Value, "hi_threashold", translate("High Threashold"), translate(""))
hi_threashold.datatype = "range(0,10000)"

low_threashold = s:option(Value, "low_threashold", translate("Low Threashold"), translate(""))
low_threashold.datatype = "range(0,10000)"

local modbus_to_iec104 = nixio.fs.access("/usr/bin/modbus-to-iec104")
if modbus_to_iec104 then
    ioa_type = s:option(ListValue, "ioa_type", "IOA Type")
    ioa_type:value("30", translate(" Single Point Information with timestamp"))
    ioa_type:value("31", translate(" Double Point Information with timestamp"))
    ioa_type:value("34", translate(" Measured Value- normalized value with timestamp"))
    ioa_type:value("35", translate(" Measured Value- scaled value with timestamp"))
    ioa_type:value("36", translate(" Measured Value- short floating point value timestamp"))


    ioa_address = s:option(Value, "ioa_address", "IOA Address")
end

local dnp3 = nixio.fs.access("/usr/bin/modbus-to-dnp3")
if dnp3 then
    dnp3_static_variation = s:option(ListValue, "dnp3_static_variation", "Default Static Variation")
    --	dnp3_static_variation:value("Group1Var0", translate("Binary Input - Any Variation"))
    dnp3_static_variation:value("Group1Var1", translate("Binary Input - Packed Format"))
    dnp3_static_variation:value("Group1Var2", translate("Binary Input - With Flags"))

    --	dnp3_static_variation:value("Group3Var0", translate("Double-bit Binary Input - Any Variation"))
    --	dnp3_static_variation:value("Group3Var1", translate("Double-bit Binary Input - Packed Format"))
    dnp3_static_variation:value("Group3Var2", translate("Double-bit Binary Input - With Flags"))

    --	dnp3_static_variation:value("Group10Var0", translate("Binary Output - Any Variation"))
    --	dnp3_static_variation:value("Group10Var1", translate("Binary Output - Packed Format"))
    dnp3_static_variation:value("Group10Var2", translate("Binary Output - Output Status With Flags"))

    --	dnp3_static_variation:value("Group30Var0", translate("Analog Input - Any Variation"))
    dnp3_static_variation:value("Group30Var1", translate("Analog Input - 32-bit With Flag"))
    dnp3_static_variation:value("Group30Var2", translate("Analog Input - 16-bit With Flag"))
    dnp3_static_variation:value("Group30Var3", translate("Analog Input - 32-bit Without Flag"))
    dnp3_static_variation:value("Group30Var4", translate("Analog Input - 16-bit Without Flag"))
    dnp3_static_variation:value("Group30Var5", translate("Analog Input - Single-precision With Flag"))
    dnp3_static_variation:value("Group30Var6", translate("Analog Input - Double-precision With Flag"))

    --	dnp3_static_variation:value("Group40Var0", translate("Analog Output Status - Any Variation"))
    dnp3_static_variation:value("Group40Var1", translate("Analog Output Status - 32-bit With Flag"))
    dnp3_static_variation:value("Group40Var2", translate("Analog Output Status - 16-bit With Flag"))
    dnp3_static_variation:value("Group40Var3", translate("Analog Output Status - Single-precision With Flag"))
    dnp3_static_variation:value("Group40Var4", translate("Analog Output Status - Double-precision With Flag"))


    --	dnp3_static_variation:value("Group41Var0", translate("Analog Output - Any Variation"))
    --	dnp3_static_variation:value("Group41Var1", translate("Analog Output - 32-bit With Flag"))
    --	dnp3_static_variation:value("Group41Var2", translate("Analog Output - 16-bit With Flag"))
    --	dnp3_static_variation:value("Group41Var3", translate("Analog Output - Single-precision With Flag"))
    --	dnp3_static_variation:value("Group41Var4", translate("Analog Output - Double-precision With Flag"))


    dnp3_event_variation = s:option(ListValue, "dnp3_event_variation", "Default Event Variation")
    --	dnp3_event_variation:value("Group2Var0", translate("Binary Input Event - Any Variation"))
    dnp3_event_variation:value("Group2Var1", translate("Binary Input Event - Without Time"))
    dnp3_event_variation:value("Group2Var2", translate("Binary Input Event - With Absolute Time"))
    dnp3_event_variation:value("Group2Var3", translate("Binary Input Event - With Relative Time"))

    --	dnp3_event_variation:value("Group4Var0", translate("Double-bit Binary Input Event - Any Variation"))
    dnp3_event_variation:value("Group4Var1", translate("Double-bit Binary Input Event - Without Time"))
    dnp3_event_variation:value("Group4Var2", translate("Double-bit Binary Input Event - With Absolute Time"))
    dnp3_event_variation:value("Group4Var3", translate("Double-bit Binary Input Event - With Relative Time"))

    --	dnp3_event_variation:value("Group11Var0", translate("Binary Output Event - Any Variation"))
    dnp3_event_variation:value("Group11Var1", translate("Binary Output Event - Output Status Without Time"))
    dnp3_event_variation:value("Group11Var2", translate("Binary Output Event - Output Status With Time"))

    --	dnp3_event_variation:value("Group32Var0", translate("Analog Input Event - Any Variation"))
    dnp3_event_variation:value("Group32Var1", translate("Analog Input Event - 32-bit With Flag"))
    dnp3_event_variation:value("Group32Var2", translate("Analog Input Event - 16-bit With Flag"))
    dnp3_event_variation:value("Group32Var3", translate("Analog Input Event - 32-bit With Flag and Time"))
    dnp3_event_variation:value("Group32Var4", translate("Analog Input Event - 16-bit With Flag and Time"))
    dnp3_event_variation:value("Group32Var5", translate("Analog Input Event - Single-precision With Flag"))
    dnp3_event_variation:value("Group32Var6", translate("Analog Input Event - Double-precision With Flag "))
    dnp3_event_variation:value("Group32Var7", translate("Analog Input Event - Single-precision With Flag and Time"))
    dnp3_event_variation:value("Group32Var8", translate("Analog Input Event - Double-precision With Flag and Time"))

    --	dnp3_event_variation:value("Group42Var0", translate("Analog Output Event - Any Variation"))
    dnp3_event_variation:value("Group42Var1", translate("Analog Output Event - 32-bit With Flag"))
    dnp3_event_variation:value("Group42Var2", translate("Analog Output Event - 16-bit With Flag"))
    dnp3_event_variation:value("Group42Var3", translate("Analog Output Event - 32-bit With Flag and Time"))
    dnp3_event_variation:value("Group42Var4", translate("Analog Output Event - 16-bit With Flag and Time"))
    dnp3_event_variation:value("Group42Var5", translate("Analog Output Event - Single-precision With Flag"))
    dnp3_event_variation:value("Group42Var6", translate("Analog Output Event - Double-precision With Flag "))
    dnp3_event_variation:value("Group42Var7", translate("Analog Output Event - Single-precision With Flag and Time"))
    dnp3_event_variation:value("Group42Var8", translate("Analog Output Event - Double-precision With Flag and Time"))

    class = s:option(ListValue, "dnp3_class", translate("Class"))
    class:value("Class0", translate("Class 0"))
    class:value("Class1", translate("Class 1"))
    class:value("Class2", translate("Class 2"))
    class:value("Class3", translate("Class 3"))

    dnp3_index = s:option(Value, "dnp3_index", "Index")
    dnp3_index.datatype = "integer"

    dnp3_deadband = s:option(Value, "dnp3_deadband", "DNP3 Deadband")
    dnp3_deadband.datatype = "integer"
end

send_alarm = s:option(ListValue, "send_alarm", translate("Send Alarm"))
send_alarm:value("high", translate("High"))
send_alarm:value("low", translate("Low"))
send_alarm:value("high_or_low", translate("High or Low"))
send_alarm:value("none", translate("None"))
send_alarm.default = "none"
send_alarm.rmempty = false

send_alarm_stat = s:option(ListValue, "send_alarm_stat", translate("Send Status Of Alarm By SMS/EMAIL"))
send_alarm_stat:value("1", translate("Yes"))
send_alarm_stat:value("0", translate("No"))
send_alarm_stat.default = "0"

notification_type = s:option(ListValue, "notification_type", translate("Notification Type"))
notification_type:value("sms", translate("SMS"))
notification_type:depends("send_alarm_stat","1")

notification_recipient = s:option(Value, "notification_recipient", translate("Notification Recepient (Mobile Number/Email Id)"))
notification_recipient.datatype = "string"
notification_recipient:depends("send_alarm_stat","1")

return m
